#include ".\..\inc\ch554.h"
#include ".\..\inc\mytypes.h"
#include ".\..\inc\device.h"
#include ".\..\inc\midi.h"

bit     bActiveSense;    // flag for signaling active sense timeout
bit     bTx0Empty;       // flag for signaling no more data in out fifo

uint8_t sensing;
uint8_t rx0led;  
uint8_t tx0led;

/** Init Timer2 as multipurpose timeout

    Set Timer 2 to autoreload with 2 ms @ 6 MHz
    and start the timer
*/
void IntitTimeout(void)
{
    T2CON   =  0x00;       
    T2MOD  &= ~bT2_CLK ;                         // T2 clock = fsys / 12 
    RCAP2   =  0x10000-(FREQ_SYS / 12 / 500 *2); //2 ms timer
    T2COUNT =  0x10000-(FREQ_SYS / 12 / 500 *2);
    TF2     =  0;
    TR2     =  1;        // T2 run
    ET2     =  1;        // enable the irq
}

/** Irq function for Timer 2

    Used to switch off the traffic leds and for active sensing 
*/


void TimeoutIrq (void) interrupt INT_NO_TMR2
{
   TF2=0;
   if (rx0led != 255) // Timeout on
   {
      rx0led --;
      if (rx0led==0) LED1=LED_OFF;
   }

   if (tx0led != 255) // Timeout on
   {
      tx0led --;
      if (tx0led==0) LED2=LED_OFF;
   }
   sensing++;
   if (sensing>=SENSE_TIMEOUT) bActiveSense=1;
}

/** Init uart1 to be used as Midi port

    Setup uart1 to 8N1 @ 31k250 baud. Enable the uart and set
    irq prioity to high.
*/
void InitUart1(void)
{
   U1SM0    = 0; // 8 bit
   U1SMOD   = 1; 
   U1REN    = 1;
   SBAUD1   = (uint8_t) (256 - (FREQ_SYS/16/31250));   
   IE_UART1 = 1;
   //IP_EX   |= bIP_UART1;                  // high priority for uart 1
   ES       = 0;
   bTx0Empty= 1;
}

void InitUart0(void)
{
   RCLK  = 0; TCLK = 0;  // use T1 for baudrate
   PCON |=  SMOD;
   // setup T1 as baudrate generator
   TMOD   = (TMOD &~ MASK_T1_MOD) | bT1_M1; // automatic reload 8 bit timer
   T2MOD |=  bTMR_CLK | bT1_CLK;            // runing at fsys / 16 or fsys / 32 

   TH1    = (uint8_t) (256 - (FREQ_SYS/16/31250));   
   TR1    = 1;   
   TI     = 1;
   SCON   = 0x50;                         // mode 2 8bit var baudrate multi processor off  
   ES     = 1;                            // enable the irq
   //PS     = 1;                          // high priority for uart0
   IE_UART1 = 0;
   bTx0Empty= 1;
}


/** Interrupt function for the uart1

    For recieving data a put into the rx ringbuffer. Bufferindex is limited 
    and counter representing the amound of data is incremented.
    Sending data is comming from the tx ringbuffer. For tx there is one 
    additional event because U1TI indicates the tx byte is send. If level
    shows that there are no more data a flag is set which will be reset by 
    the usb irq after new data are put into the ringbuffer.
*/

void Uart1_ISR(void) interrupt INT_NO_UART1
{
	 // rx irq 
   if(U1RI)
   {
      MidiRxBuffer[MidiRxCtl.rd++] = SBUF1;
      MidiRxCtl.rd &= MIDIBUF_MASK; // limit the length
      MidiRxCtl.level++;
      LED1=LED_ON;
      rx0led = LED_TIMEOUT;
      U1RI = 0;
   }

   // tx irq
   if (U1TI)
   {
   	  U1TI=0;
   	  if (MidiTxCtl.level)
   	  {	
   	     SBUF1=MidiTxBuffer[MidiTxCtl.wr++];
   	     MidiTxCtl.wr &= MIDIBUF_MASK;
   	     MidiTxCtl.level--;
   	     LED2=LED_ON;
   	     tx0led = LED_TIMEOUT;
   	  }
   	  else bTx0Empty = 1;

      // unlock BulkOut for the next packet
      if( (bEP2Lock) && (MidiTxCtl.level < MIDIBUF_LOWER) )
      {
         bEP2Lock=0;
         UEP2_CTRL = UEP2_CTRL & ~ MASK_UEP_R_RES | UEP_R_RES_ACK;
      }  
   	  sensing = 0;
   }	
}

/** Interrupt function for the uart0

    same functionality as Uat1Irq() but does access the first Uart regs 
*/
void Uart0_ISR(void) interrupt INT_NO_UART0
{
	 // rx irq 
   if(RI)
   {
      MidiRxBuffer[MidiRxCtl.rd++] = SBUF;
      MidiRxCtl.rd &= MIDIBUF_MASK; // limit the length
      MidiRxCtl.level++;
      LED1=LED_ON;
      rx0led = LED_TIMEOUT;
      RI = 0;
   }

   // tx irq
   if (TI)
   {
   	  TI=0;
   	  if (MidiTxCtl.level)
   	  {	
   	     SBUF=MidiTxBuffer[MidiTxCtl.wr++];
   	     MidiTxCtl.wr &= MIDIBUF_MASK;
   	     MidiTxCtl.level--;
   	     LED2=LED_ON;
   	     tx0led = LED_TIMEOUT;
   	  }
   	  else bTx0Empty = 1;

      // unlock BulkOut for the next packet
      if( (bEP2Lock) && (MidiTxCtl.level < MIDIBUF_LOWER) )
      {
         bEP2Lock=0;
         UEP2_CTRL = UEP2_CTRL & ~ MASK_UEP_R_RES | UEP_R_RES_ACK;
      }
   	  sensing = 0;
   }	
}

